<?php

namespace App\Controllers\Seguridad;

use App\Controllers\BaseController;
use App\Models\Seguridad\Usuarios as SeguridadUsuarios;
use Config\Services;

class Usuarios extends BaseController
{

    public function __construct()
    {
        // helper('application');
        // Config('Config\\Enums\\ActionsEnums');     
    }

    public function index()
    {
        // if(!validate_permissions(session()->get('rol_id'), 8, ActionsEnums::CONSULTAR)){
        //     return redirect()->to(base_url('pages/error-403'));
        // }
        if(session()->get('rol') != 101){
            return redirect()->to(base_url('pages/error-403'));
        }

        $Usuarios = new SeguridadUsuarios();

        $data = [
            "usuarios" => $Usuarios->listar(),
        ];

        return view("seguridad/usuarios", $data);
    }

    public function crear()
    {
        if(session()->get('rol') != 101){
            return redirect()->to(base_url('pages/error-403'));
        }

        helper(['form']);
        $validation = service('validation');
        // if(!validate_permissions(session()->get('rol_id'), 8, ActionsEnums::CREAR)){
        // return redirect()->to(base_url('pages/error-403'));
        // }

        $rules = [
            'nombre'         => [
                'label' => 'persona',
                'rules' => 'required|min_length[4]|max_length[100]',
                'errors' => [
                    'required' => 'El campo {field} es requerido',
                    'min_length' => 'El campo {field} debe tener una longitud mínima de {param} caracteres.',
                    'min_length' => 'El campo {field} debe tener una longitud máxima de {param} caracteres',
                ],
            ],
            'correo'         => [
                'label' => 'correo electrónico',
                'rules' => 'required|min_length[4]|max_length[100]|valid_email|is_unique[users.user_email]',
                'errors' => [
                    'required' => 'El campo {field} es requerido',
                    'min_length' => 'El campo {field} debe tener una longitud mínima de {param} caracteres.',
                    'min_length' => 'El campo {field} debe tener una longitud máxima de {param} caracteres',
                    'valid_email' => 'El correo ingresado es inválido.',
                    'is_unique' => 'El campo {field} ya se encuentra registrado en el sistema'
                ],
            ],

        ];
        $_POST = [
            "user_name" => $_POST['nombre'],
            "status" => $_POST['estatus_id'],
            "user_email" => $_POST['correo'],
            "rol" => 102
        ];

        $formData = $_POST;

        if (!$this->validate($rules)) {
            $session = session();
            $validation->withRequest($this->request)->run();
            $session->setFlashdata('errors', $validation->getErrors());
            return redirect()->to(base_url() . '/usuarios');
        }

        $usuario = new SeguridadUsuarios();
        $respuesta = $usuario->insertar($formData);

        if ($respuesta > 0) {
            session()->setFlashdata('msg-success', 'Operación realizada con éxito');
            return redirect()->to(base_url() . '/usuarios');
        } else {
            session()->setFlashdata('msg-error', 'Ocurrio un error en la operación, favor de validar la información');
            return redirect()->to(base_url() . '/usuarios');
        }
    }

    public function actualizar()
    {
        if(session()->get('rol') != 101){
            return redirect()->to(base_url('pages/error-403'));
        }

        helper(['form']);
        $validation = service('validation');
        $is_unique = "";

        // if(!validate_permissions(session()->get('rol_id'), 8, ActionsEnums::EDITAR)){
        //     return redirect()->to(base_url('pages/error-403'));
        // }

        $id = $_POST['idUpdate'];
        $usuario = new SeguridadUsuarios();
        $emailValidate = $usuario->where('user_id', $id)->first();

        if (!empty($emailValidate)) {
            if ($emailValidate['user_email'] != trim($_POST['correoUpdate'])) {
                $is_unique = "|is_unique[users.user_email]";
            }
        }

        $rules = [
            'idUpdate'                   => [
                'rules' => 'required',
                'errors' => [
                    'required' => 'Ocurrió un error con el usuario seleccionado, favor de intentar seleccionarlo de nuevo',
                ],
            ],
            'nombreUpdate'         => [
                'label' => 'persona',
                'rules' => 'required|min_length[4]|max_length[100]',
                'errors' => [
                    'required' => 'El campo {field} es requerido',
                    'min_length' => 'El campo {field} debe tener una longitud mínima de {param} caracteres.',
                    'min_length' => 'El campo {field} debe tener una longitud máxima de {param} caracteres',
                ],
            ],

            'correoUpdate'         => [
                'label' => 'correo electrónico',
                'rules' => 'required|min_length[4]|max_length[100]|valid_email' . $is_unique,
                'errors' => [
                    'required' => 'El campo {field} es requerido',
                    'min_length' => 'El campo {field} debe tener una longitud mínima de {param} caracteres.',
                    'min_length' => 'El campo {field} debe tener una longitud máxima de {param} caracteres',
                    'valid_email' => 'El correo ingresado es inválido.',
                ],
            ],

        ];

        if (!$this->validate($rules)) {
            $session = session();
            $validation->withRequest($this->request)->run();
            $session->setFlashdata('errors', $validation->getErrors());
            return redirect()->to(base_url() . '/usuarios');
        }

        $formData = [
            'user_id' => $_POST['idUpdate'],
            'user_name' => $_POST['nombreUpdate'],
            'status' => $_POST['estatus_idUpdate'],
            'user_email' => $_POST['correoUpdate'],
            "rol" => 102
        ];

        $respuesta = $usuario->actualizar($formData, $id);

        if ($respuesta > 0) {
            session()->setFlashdata('msg-success', 'Operación realizada con éxito');
            return redirect()->to(base_url() . '/usuarios');
        } else {
            session()->setFlashdata('msg-error', 'Ocurrio un error en la operación, favor de validar la información');
            return redirect()->to(base_url() . '/usuarios');
        }
    }

    public function eliminar($id)
    {
        // if(!validate_permissions(session()->get('rol_id'), 8, ActionsEnums::ELIMINAR)){
        //     return redirect()->to(base_url('/pages/error-403'));
        // }

        if(session()->get('rol') != 101){
            return redirect()->to(base_url('pages/error-403'));
        }

        $usuario = new SeguridadUsuarios();
        $usuarioValidate = $usuario->where('user_id', $id)->first();
        if ($usuarioValidate['status'] != 8) {
            $usuarioValidate['status'] = 8;
        } else {
            $usuarioValidate['status'] = 1;
        }

        $respuesta = $usuario->actualizar($usuarioValidate, $id);

        if ($respuesta) {
            session()->setFlashdata('msg-success', 'Operación realizada con éxito');
            return redirect()->to(base_url() . '/usuarios');
        } else {
            session()->setFlashdata('msg-error', 'Ocurrio un error en la operación, favor de validar la información');
            return redirect()->to(base_url() . '/usuarios');
        }
    }

    public function email($id)
    {
        // if(!validate_permissions(session()->get('rol_id'), 8, ActionsEnums::CREAR) && !validate_permissions(session()->get('rol_id'), 8, ActionsEnums::EDITAR)){
        //     return redirect()->to(base_url('/pages/error-403'));
        // }

        if(session()->get('rol') != 101){
            return redirect()->to(base_url('pages/error-403'));
        }

        $usuario = new SeguridadUsuarios();
        $usuarioValidate = $usuario->where('user_id', $id)->first();
        // if (!$usuarioValidate['esenviado']) {
        //     $usuarioValidate['esenviado'] = true;
        // }

        if (!empty($usuarioValidate)) {
            $email = \Config\Services::email();

            // $email->setFrom('no-reply@inndot.com', 'INNDOT - Piensa soluciones');
            $email->setFrom('soporte@kodit.com.mx', 'KODIT');
            $email->setTo($usuarioValidate['user_email']);

            $email->setSubject('Invitación de usuario');

            $data = [
                "url_app" => base_url(),
                "title" => "KODIT",
                "url" => base_url('/update-password'),
                "id" => $usuarioValidate['user_id'],
                "user" => $usuarioValidate['user_name'],
            ];

            $view = \Config\Services::renderer();

            $html = $view->setData($data, 'raw')->render('Mails/update-forgot', $data, true);

            $email->setMessage($html);

            // var_dump($email);
            // return;
            if ($email->send()) {
                Session()->setFlashdata('msg-success', 'Se ha envíado el correo de invitación a ' . $usuarioValidate['user_email']);
                $respuesta = $usuario->actualizar($usuarioValidate, $id);
            } else {
                Session()->setFlashdata('msg-success', 'No se pudo enviar el correo de invitación a ' . $usuarioValidate['user_email'] . ', favor de validar la información.');
            }

            return redirect()->to(base_url('/usuarios'));
        }
    }

    public function update_password($id)
    {
        $userModel =  new SeguridadUsuarios();
        //$time = openssl_decrypt($this->request->getGet('t'), "AES-128-CTR", "date", 0, '1234567891011121');
        $data = $userModel->where('user_id', openssl_decrypt($id, "AES-128-CTR", "user_id", 0, '1234567891011121'))->first();

        if (!isset($data)) {
            return view('pages/error-404');
        }

        // if(!empty($data['password']) || $data['esEnviado'] != 0){
        //     return view('pages/error-404');
        // }

        $data['url_recovery'] = base_url('/update-password') . '/' . $id;
        return view('login/recovery-password', $data);
    }

    // public function cvsimport() 
    // {
    //     if(isset($_POST["submit"]))
    // 	{
    //         $texto = "";
    //         $operacion = 1;
    // 		$file = $_FILES['file']['tmp_name'];
    //         if(!$file){
    //             session()->setFlashdata('msg-error', 'Ocurrio un error en la operación, favor de adjuntar el archivo');
    //             return redirect()->to(base_url('/usuarios'));
    //         }
    // 		$handle = fopen($file, "r");
    // 		$f = fopen($file, "r");
    // 		$c = 0;
    //         $boolean = (count(fgetcsv($f,0,',','"')) > 1 ? true : false);

    //         if($boolean)
    //         {
    //             while( ($filesop = fgetcsv($handle, 10000, ",", '"')) !== false )
    //             {	
    //                 if($c<>0)
    //                 {
    //                     $persona = $filesop[0];
    //                     $rol = $filesop[1];
    //                     $estatus = $filesop[2];
    //                     $correo = $filesop[3];
    //                     $posicion = $filesop[4];
    //                     $bu = $filesop[5];
    //                     $empresa = $filesop[6];
    //                     $tipo_contrato = $filesop[7];
    //                     $c_hora_inndot = $filesop[8];
    //                     $c_hora_empresa = $filesop[9];

    //                     if(!$persona){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un nombre de persona en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(strlen($persona) < 4){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un nombre de persona de longitud mínima de 4 caracteres en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(strlen($persona) > 100){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un nombre de persona de longitud máxima de 100 caracteres en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$rol){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un rol en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$estatus){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un estatus en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$correo){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un correo en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$posicion){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar una posición en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$bu){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un BU en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$tipo_contrato){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un tipo de contrato en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }

    //                     $roles_model = new Roles();
    //                     $rol_id = $roles_model->get_rol_id($rol);
    //                     if(!$rol_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el rol en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $estatus_model = new EstatusModel();
    //                     $estatus_id = $estatus_model->get_estatus_id($estatus);
    //                     if(!$estatus_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el estatus en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$this->validateEmail($correo))
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el correo en la fila $c no es válido, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $usuarios_model = new SeguridadUsuarios();
    //                     $usuario_correo = $usuarios_model->get_usuario_correo($correo);
    //                     if($usuario_correo)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el correo en la fila $c ya existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $posicion_model = new PosicionesModel();
    //                     $posicion_id = $posicion_model->get_posicion_id($posicion);
    //                     if(!$posicion_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "la posición en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $bu_model = new BuModel();
    //                     $bu_id = $bu_model->get_bu_id($bu);
    //                     if(!$bu_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el BU en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $tipocontrato_model = new TipoContratoModel();
    //                     $tipocontrato_id = $tipocontrato_model->get_tipocontrato_id($tipo_contrato);
    //                     if(!$tipocontrato_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el tipo de contrato en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$c_hora_inndot){
    //                         $c_hora_inndot = 0;
    //                     }
    //                     if(!$c_hora_empresa){
    //                         $c_hora_empresa = 0;
    //                     }
    //                     $costohoratotal = $c_hora_inndot + $c_hora_empresa;
    //                     $esenviado = 0;
    //                     $creado_por = Session()->get('id');

    //                     $datos = [
    //                         'id' => 'null',
    //                         'nombre' => $persona,
    //                         'rol_id' => $rol_id['id'],
    //                         'estatus_id' => $estatus_id['id'],
    //                         'correo' => $correo,
    //                         'posicion_id' => $posicion_id['id'],
    //                         'bu_id' => $bu_id['id'],
    //                         'empresa' => $empresa,
    //                         'tipocontrato_id' => $tipocontrato_id['id'],
    //                         'costohoraorigen' => $c_hora_inndot,
    //                         'costohoraproveedor' => $c_hora_empresa,
    //                         'costohoratotal' => $costohoratotal,
    //                         'esenviado' => $esenviado,
    //                         'creado_por' => $creado_por
    //                     ];
    //                     $usuarios_model->insertar($datos);
    //                 }
    //                 $c = $c + 1;
    //             }
    //         }
    //         else
    //         {
    //             while( ($filesop = fgetcsv($handle, 10000, ";", '"')) !== false )
    //             {	
    //                 if($c<>0)
    //                 {
    //                     $persona = $filesop[0];
    //                     $rol = $filesop[1];
    //                     $estatus = $filesop[2];
    //                     $correo = $filesop[3];
    //                     $posicion = $filesop[4];
    //                     $bu = $filesop[5];
    //                     $empresa = $filesop[6];
    //                     $tipo_contrato = $filesop[7];
    //                     $c_hora_inndot = $filesop[8];
    //                     $c_hora_empresa = $filesop[9];

    //                     if(!$persona){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un nombre de persona en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(strlen($persona) < 4){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un nombre de persona de longitud mínima de 4 caracteres en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(strlen($persona) > 100){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un nombre de persona de longitud máxima de 100 caracteres en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$rol){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un rol en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$estatus){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un estatus en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$correo){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un correo en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$posicion){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar una posición en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$bu){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un BU en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$tipo_contrato){
    //                         $operacion = 0;
    //                         $texto = "debe proporcionar un tipo de contrato en la fila $c, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }

    //                     $roles_model = new Roles();
    //                     $rol_id = $roles_model->get_rol_id($rol);
    //                     if(!$rol_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el rol en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $estatus_model = new EstatusModel();
    //                     $estatus_id = $estatus_model->get_estatus_id($estatus);
    //                     if(!$estatus_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el estatus en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$this->validateEmail($correo))
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el correo en la fila $c no es válido, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $usuarios_model = new SeguridadUsuarios();
    //                     $usuario_correo = $usuarios_model->get_usuario_correo($correo);
    //                     if($usuario_correo)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el correo en la fila $c ya existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $posicion_model = new PosicionesModel();
    //                     $posicion_id = $posicion_model->get_posicion_id($posicion);
    //                     if(!$posicion_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "la posición en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $bu_model = new BuModel();
    //                     $bu_id = $bu_model->get_bu_id($bu);
    //                     if(!$bu_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el BU en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     $tipocontrato_model = new TipoContratoModel();
    //                     $tipocontrato_id = $tipocontrato_model->get_tipocontrato_id($tipo_contrato);
    //                     if(!$tipocontrato_id)
    //                     {
    //                         $operacion = 0;
    //                         $texto = "el tipo de contrato en la fila $c no existe, revise la información, ya no se insertaron los demás datos";
    //                         break;
    //                     }
    //                     if(!$c_hora_inndot){
    //                         $c_hora_inndot = 0;
    //                     }
    //                     if(!$c_hora_empresa){
    //                         $c_hora_empresa = 0;
    //                     }
    //                     $costohoratotal = $c_hora_inndot + $c_hora_empresa;
    //                     $esenviado = 0;
    //                     $creado_por = Session()->get('id');

    //                     $datos = [
    //                         'id' => 'null',
    //                         'nombre' => $persona,
    //                         'rol_id' => $rol_id['id'],
    //                         'estatus_id' => $estatus_id['id'],
    //                         'correo' => $correo,
    //                         'posicion_id' => $posicion_id['id'],
    //                         'bu_id' => $bu_id['id'],
    //                         'empresa' => $empresa,
    //                         'tipocontrato_id' => $tipocontrato_id['id'],
    //                         'costohoraorigen' => $c_hora_inndot,
    //                         'costohoraproveedor' => $c_hora_empresa,
    //                         'costohoratotal' => $costohoratotal,
    //                         'esenviado' => $esenviado,
    //                         'creado_por' => $creado_por
    //                     ];
    //                     $usuarios_model->insertar($datos);
    //                 }
    //                 $c = $c + 1;
    //             }
    //         }

    //         if($operacion > 0){
    //             session()->setFlashdata('msg-success', 'Operación realizada con éxito');
    //             return redirect()->to(base_url('/usuarios'));
    //         }else{
    //             session()->setFlashdata('msg-error', "Ocurrio un error en la operación, $texto");
    //             return redirect()->to(base_url('/usuarios'));
    //         }
    //     }

    // }   

    // public function usuarios_cvs() 
    // {
    //     header('Content-Encoding: utf-8');
    // 	header("Content-Type: text/CSV; charset=UTF-8");
    //     $output = fopen('php://output', 'w');
    //     fputs( $output, "\xEF\xBB\xBF" );
    //     $encabezados = ["Persona","Rol","Estatus","Correo","Posición","BU","Empresa","Tipo de contrato","Costo por hora - inndot","Costo por hora - Empresa"];
    //     fputcsv($output,$encabezados, ";");
    //     $renglones = ["Jessica Ortíz","Administrador","Activo","correo@ejemplo.com","PMO SAP","INNDOT","INNDOT","Tiempo completo","100.50","200.50"];
    //     fputcsv($output,$renglones, ";");

    //     fclose($output);
    //     exit;
    // }

    // private function validateEmail($email) {
    //     if(filter_var($email, FILTER_VALIDATE_EMAIL)) {
    //         return true;
    //     }
    //     else {
    //         return false;
    //     }
    // }


}
